#include "example.h"

static void usage( char *pgmName );

#define OPT_ARG_LIST        "hd:i:o:v:"

/********************************************************
**
** Test Module: Poke.c
**
** Purpose:     Set a value at a given offset from the
**              base register on the board.
**
**
*********************************************************/

int main (int argc, char *argv[])
{
    TPRO_BoardObj   *hnd = NULL;
    TPRO_MemObj     MemObj;
    int             rc;
    int             ret = 0;
    int             devIdx = DEFAULT_DEV_NUM;
    char            *devName = DEFAULT_DEV;
    int             optRc;
    char            fullDevName[MAX_DEVICE_PATH_LENGTH];
    char            *errMsg;
    char            *pgmname = argv[0];
    int             foundRequiredParam = 0;

    /*
     * init the Memory object
     */
    MemObj.offset = 0;
    MemObj.value = 0;

    opterr = 0;
    while ( (optRc = getopt (argc, argv, OPT_ARG_LIST)) != -1 )
    {
        switch ( optRc )
        {
            case 'd' :
                devName = optarg;
                break;
            case 'i' :
                devIdx = atoi(optarg);
                break;
            case 'o' :
                MemObj.offset = (uint16_t) strtoul(optarg, NULL, 16);
                foundRequiredParam++ ;
                break;
            case 'v' :
                MemObj.value = (uint16_t) strtoul(optarg, NULL, 16);
                foundRequiredParam++;
                break;
            case 'h' :
            default:
                usage( pgmname );
                return (1);
        }
    }

    /*
     * If leftover args or offset and value not specified,
     * let user know usage.
     */
    if ( (optind != argc) || (foundRequiredParam != 2) )
    {
        usage( pgmname );
        return (1);
    }

    /*
     * construct the device name
     */
    (void)sprintf_s( fullDevName, "%s%s%d", DEVICE_ROOT, devName, devIdx );

    /*
     * open the device
     */
    if ( (rc = TPRO_open(&hnd, fullDevName)) != TPRO_SUCCESS )
    {
        /* 
         * if errors, close it
         */
        printf("%s: Could not open <%s> : rc <%d>\n", 
                        pgmname, fullDevName, rc);
        return (1);
    }

    /*
     * get the current time
     */
    if ( (rc = TPRO_poke(hnd, &MemObj)) != TPRO_SUCCESS )
    {
        switch (rc)
        {
            case TPRO_HANDLE_ERR:
                errMsg = "TPRO_HANDLE_ERR";
                break;
            case TPRO_COMM_ERR:
                errMsg = "TPRO_COMM_ERR";
                break;
            default:
                errMsg = "Unknown error";
                break;
        }
        (void) printf("%s: <%s> %s rc: <%d>.\n", 
                    pgmname, fullDevName, errMsg, rc);

        ret = 1;

    }
    else
    {
        /*
         * success:  print out confirmation
         */
        (void)printf("<%s> %s: reg offset 0x<%04x> value 0x<%04x> written.\n",
                    fullDevName, pgmname, MemObj.offset, MemObj.value); 
    }

    /*
     * close the handle to the board
     */
    if ( (rc = TPRO_close(hnd)) != TPRO_SUCCESS )
    {
        (void) printf("%s: error closing <%s> rc: <%d>\n", 
                pgmname, fullDevName, rc );
        ret = 1;
    }

    return (ret);
}

void usage( char *pgmName )
{
    (void)printf("Usage: %s -o [HEX_OFFSET] -v [HEX_VALUE] [-h] [-d NAME] [-i INDEX] \n\n", pgmName );
    (void)printf("\t-h   help message\n");
    (void)printf("\t-o   offset (in hex) from base address\n");
    (void)printf("\t-v   value (in hex) to write at the offset \n");
    (void)printf("\t-d   device name (e.g. %s)\n", DEFAULT_DEV);
    (void)printf("\t-i   device index (e.g. 0,1,..)\n");
    (void)printf("\t\tsuch that combined forms: %s0\n\n", DEFAULT_DEV);
}
