#include "example.h"

static void usage( char *pgmName );

#define OPT_ARG_LIST        "hd:i:"

/********************************************************
**
** Test Module: SynchStatus.c
**
** Purpose:     Report synch status of the board.
**              (freewheel vs sync-to-input)
**
*********************************************************/

int main (int argc, char *argv[])
{
    TPRO_BoardObj   *hnd = NULL;
    int             rc;
    int             ret = 0;
    int             devIdx = DEFAULT_DEV_NUM;
    char            *devName = DEFAULT_DEV;
    int             optRc;
    char            fullDevName[MAX_DEVICE_PATH_LENGTH];
    char            *errMsg;
    char            *pgmname = argv[0];
    uint8_t         status = TPRO_SYNC_STAT_FREE;

    opterr = 0;
    while ( (optRc = getopt (argc, argv, OPT_ARG_LIST)) != -1 )
    {
        switch ( optRc )
        {
            case 'd' :
                devName = optarg;
                break;
            case 'i' :
                devIdx = atoi(optarg);
                break;
            case 'h' :
            default:
                usage( pgmname );
                return (1);
        }
    }

    /*
     * If leftover args, then command is misunderstood...
     * so let user know.
     */
    if ( optind != argc )
    {
        usage( pgmname );
        return (1);
    }

    /*
     * construct the device name
     */
    (void)sprintf_s( fullDevName, MAX_DEVICE_PATH_LENGTH, "%s%s%d", DEVICE_ROOT, devName, devIdx );

    /*
     * open the device
     */
    if ( (rc = TPRO_open(&hnd, fullDevName)) != TPRO_SUCCESS )
    {
        /* 
         * if errors, close it
         */
        printf("%s: Could not open <%s> : rc <%d>\n", 
                        pgmname, fullDevName, rc);
        return (1);
    }

    /*
     * Set the synch flag
     */
    if ( (rc = TPRO_synchStatus(hnd, &status)) != TPRO_SUCCESS )
    {
        switch (rc)
        {
            case TPRO_HANDLE_ERR:
                errMsg = "TPRO_HANDLE_ERR";
                break;
            case TPRO_COMM_ERR:
                errMsg = "TPRO_COMM_ERR";
                break;
            default:
                errMsg = "Unknown error";
                break;
        }
        (void) printf("%s: <%s> %s rc: <%d>.\n", 
                    pgmname, fullDevName, errMsg, rc);

        ret = 1;

    }
    else
    {
        /*
         * print out status determined 
         */
        (void)printf("<%s> %s: < %s >\n", fullDevName, pgmname,
                ((status == TPRO_SYNC_STAT_FREE) ? "Freewheeling" : "Sync-To-Input" ) );
    }

    /*
     * close the handle to the board
     */
    if ( (rc = TPRO_close(hnd)) != TPRO_SUCCESS )
    {
        (void) printf("%s: error closing <%s> rc: <%d>\n", 
                pgmname, fullDevName, rc );
        ret = 1;
    }

    return (ret);
}

void usage( char *pgmName )
{
    (void)printf("Usage: %s [-h] [-d NAME] [-i INDEX] \n\n", pgmName );
    (void)printf("\t-h   help message\n");
    (void)printf("\t-d   device name (e.g. %s)\n", DEFAULT_DEV);
    (void)printf("\t-i   device index (e.g. 0,1,..)\n");
    (void)printf("\t\tsuch that combined forms: %s0\n\n", DEFAULT_DEV);
}
